'use strict';

/**
 * Pagean test framework.
 *
 * @module pagean
 */
const puppeteer = require('puppeteer');

const testLogger = require('./lib/logger');
const testReporter = require('./lib/reporter');
const { ...testFunctions } = require('./lib/tests');
const { createLinkChecker } = require('./lib/link-utils');

/**
 * @typedef  {object} PageOptions
 * @property {number} retryCount  - The number of times to retry navigation.
 * @property {number} timeout     - The timeout for navigation.
 */
const defaultPageOptions = {
    retryCount: 2,
    timeout: 60_000
};

/**
 * Opens the specified URL in the given page, with retry logic.
 *
 * @param {string}      page    The puppeteer page object.
 * @param {string}      url     The URL to navigate to.
 * @param {PageOptions} options The options for navigating to the URL.
 * @static
 * @private
 */
const gotoPage = async (page, url, options) => {
    let retryCount = 0;
    while (retryCount < options.retryCount) {
        try {
            /* eslint-disable no-await-in-loop -- loop for retry only */
            // User provided test input required.
            // nosemgrep: puppeteer-goto-injection
            await page.goto(url, {
                timeout: options.timeout,
                waitUntil: 'domcontentloaded'
            });
            /* eslint-enable no-await-in-loop -- loop for retry only */
            break;
        } catch {
            retryCount++;
            console.error(
                `Failed to navigate to ${url}. Retrying (${retryCount}/${options.retryCount})...`
            );
        }
    }
};

/* eslint-disable no-await-in-loop, max-lines-per-function --
   no-await-in-loop - tests are deliberately performed synchronously,
   max-lines-per-function - allowed to test executor */
/**
 * Executes Pagean tests as specified in config and reports results.
 *
 * @param {object} config The Pagean test configuration.
 * @static
 */
const executeAllTests = async (config) => {
    const logger = testLogger(config);
    const linkChecker = createLinkChecker();
    const browser = await puppeteer.launch(config.puppeteerLaunchOptions);

    for (const testUrl of config.urls) {
        logger.startUrlTests(testUrl.rawUrl);

        const consoleLog = [];
        const page = await browser.newPage();
        page.on('console', (message) =>
            consoleLog.push({
                location: message.location(),
                text: message.text(),
                type: message.type()
            })
        );
        await gotoPage(page, testUrl.url, defaultPageOptions);

        const testContext = {
            consoleLog,
            linkChecker,
            logger,
            page,
            urlSettings: {
                ...testUrl.settings,
                htmlHintConfig: config.htmlHintConfig
            }
        };
        for (const test of Object.keys(testFunctions)) {
            await testFunctions[test](testContext);
        }

        await page.close();
    }
    await browser.close();
    const testResults = logger.getTestResults();
    testReporter.saveReports(testResults, config.reporters);

    if (testResults.summary.failed > 0) {
        // For test harness want process to exit with error code
        // eslint-disable-next-line unicorn/no-process-exit, no-magic-numbers -- exit code
        process.exit(2);
    }
};
/* eslint-enable no-await-in-loop, max-lines-per-function -- enable */

module.exports.executeAllTests = executeAllTests;
